"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }





var _chunk22TSIM4Wjs = require('./chunk-22TSIM4W.js');

// src/core/index.ts
var _unplugin = require('unplugin');
var _magicstring = require('magic-string'); var _magicstring2 = _interopRequireDefault(_magicstring);

// src/core/parse_ID.ts
var _path = require('path');
function parse_ID(id, root = "") {
  const [file, rawQuery] = id.split("?", 2);
  const ext = _path.extname.call(void 0, file).slice(1);
  const isSfc = ext === "vue";
  const isTsx = ext === "tsx";
  const isJsx = isTsx || ext === "jsx";
  const query = Object.fromEntries(new URLSearchParams(rawQuery));
  if (query.vue != null) {
    query.vue = true;
  }
  return {
    file: file.replace(root, ""),
    isSfc,
    isTsx,
    isJsx,
    query
  };
}

// src/core/filter_ID.ts
function filter_ID(id) {
  const parsed = parse_ID(id);
  if (parsed.isJsx) {
    return true;
  }
  if (parsed.isSfc) {
    const { query } = parsed;
    if (query.type === "template") {
      return true;
    }
    return (
      // vite-plugin-vue
      !query[_chunk22TSIM4Wjs.TRACE_ID] && // rollup-plugin-vue
      !query["rollup-plugin-vue"]
    );
  }
  return false;
}

// src/core/transform_SFC.ts
var _compilerdom = require('@vue/compiler-dom');

// src/core/transform_JSX.ts
var _core = require('@babel/core');
var _parser = require('@babel/parser');
function transform_JSX(code, s, options) {
  const {
    file,
    isTsx,
    startIndex = 0,
    startLine = 1,
    startColumn = 0
  } = options;
  const plugins = ["jsx"];
  if (isTsx) {
    plugins.push("typescript");
  }
  const ast = _parser.parse.call(void 0, code, {
    sourceType: "unambiguous",
    plugins,
    startLine,
    startColumn
  });
  _core.traverse.call(void 0, ast, {
    JSXOpeningElement({ node }) {
      const nameNode = node.name;
      if (!nameNode) {
        return;
      }
      const { index, line, column } = node.loc.start;
      const name = getJSXElementName(nameNode);
      const prependIndex = index + startIndex + name.length + 1;
      s.prependLeft(prependIndex, ` ${_chunk22TSIM4Wjs.TRACE_ID}="${file}:${line}:${column}"`);
    }
  });
}
function getJSXElementName(nameNode) {
  let nameValue;
  if (_core.types.isJSXIdentifier(nameNode)) {
    nameValue = nameNode.name;
  } else if (_core.types.isJSXNamespacedName(nameNode)) {
    const { namespace, name } = nameNode;
    nameValue = `${namespace.name}:${name.name}`;
  } else {
    const nameValues = [];
    while (_core.types.isJSXMemberExpression(nameNode)) {
      nameValues.unshift(nameNode.property.name);
      nameNode = nameNode.object;
    }
    nameValues.unshift(nameNode.name);
    nameValue = nameValues.join(".");
  }
  return nameValue;
}

// src/core/transform_SFC.ts
function transform_SFC(code, s, options) {
  const { file } = options;
  const ast = _compilerdom.parse.call(void 0, code);
  _compilerdom.transform.call(void 0, ast, {
    nodeTransforms: [
      (node) => {
        if (node.type === _chunk22TSIM4Wjs.NodeTypes.ELEMENT && _chunk22TSIM4Wjs.TagTypes.includes(node.tagType)) {
          const { line, column, offset } = node.loc.start;
          const prependIndex = offset + node.tag.length + 1;
          s.prependLeft(
            prependIndex,
            ` ${_chunk22TSIM4Wjs.TRACE_ID}="${file}:${line}:${column}"`
          );
        }
      }
    ]
  });
  const jsxOpts = resolveJsxOptions(ast);
  if (jsxOpts) {
    transform_JSX(jsxOpts.code, s, _chunk22TSIM4Wjs.__spreadProps.call(void 0, _chunk22TSIM4Wjs.__spreadValues.call(void 0, {}, jsxOpts), {
      file
    }));
  }
}
function resolveJsxOptions(ast) {
  var _a;
  const scriptNode = ast.children.find(
    (node) => node.tag === "script"
  );
  if (!scriptNode)
    return;
  const codeNode = scriptNode.children[0];
  if (!codeNode)
    return;
  const langProp = scriptNode.props.find(
    (prop) => prop.name === "lang"
  );
  if (!langProp)
    return;
  const lang = (_a = langProp.value) == null ? void 0 : _a.content;
  const isTsx = lang === "tsx";
  const isJsx = isTsx || lang === "jsx";
  if (isJsx) {
    const { offset, line, column } = codeNode.loc.start;
    return {
      isTsx,
      startIndex: offset,
      startLine: line,
      startColumn: column,
      code: codeNode.content
    };
  }
}

// src/core/index.ts
var unpluginFactory = (options = {}) => {
  if (process.env.NODE_ENV !== "development") {
    return {
      name: "unplugin-vue-source"
    };
  }
  const { root = process.cwd(), sourceMap = false } = options;
  return {
    name: "unplugin-vue-source",
    enforce: "pre",
    transformInclude: filter_ID,
    transform(code, id) {
      const s = new (0, _magicstring2.default)(code);
      const parsed = parse_ID(id, root);
      if (parsed.isSfc) {
        transform_SFC(code, s, parsed);
      } else if (parsed.isJsx) {
        transform_JSX(code, s, parsed);
      }
      return {
        code: s.toString(),
        map: sourceMap ? s.generateMap() : null
      };
    }
  };
};
var core_default = /* @__PURE__ */ _unplugin.createUnplugin.call(void 0, unpluginFactory);



exports.core_default = core_default;
